/*
 * This file was generated by qdbusxml2cpp version 0.8
 * Source file was Cache.xml
 *
 * qdbusxml2cpp is Copyright (C) The Qt Company Ltd. and other contributors.
 *
 * This is an auto-generated file.
 * This file may have been hand-edited. Look for HAND-EDIT comments
 * before re-generating it.
 */

#ifndef CACHE_ADAPTOR_H
#define CACHE_ADAPTOR_H

#include <QtCore/QObject>
#include <QtDBus/QtDBus>
#include "QtGui/private/qspi_struct_marshallers_p.h"
#include <QtCore/qcontainerfwd.h>

namespace QtGuiPrivate {

/*
 * Adaptor class for interface org.a11y.atspi.Cache
 */
class CacheAdaptor: public QDBusAbstractAdaptor
{
    Q_OBJECT
    Q_CLASSINFO("D-Bus Interface", "org.a11y.atspi.Cache")
    Q_CLASSINFO("D-Bus Introspection", ""
"  <interface name=\"org.a11y.atspi.Cache\">\n"
"    <method name=\"GetItems\">\n"
"      <arg direction=\"out\" type=\"a((so)(so)a(so)assusau)\" name=\"nodes\"/>\n"
"      <annotation value=\"QSpiAccessibleCacheArray\" name=\"org.qtproject.QtDBus.QtTypeName.Out0\"/>\n"
"    </method>\n"
"    <signal name=\"AddAccessible\">\n"
"      <arg type=\"((so)(so)a(so)assusau)\" name=\"nodeAdded\"/>\n"
"      <annotation value=\"QSpiAccessibleCacheItem\" name=\"org.qtproject.QtDBus.QtTypeName.Out0\"/>\n"
"    </signal>\n"
"    <signal name=\"RemoveAccessible\">\n"
"      <arg type=\"(so)\" name=\"nodeRemoved\"/>\n"
"      <annotation value=\"QSpiObjectReference\" name=\"org.qtproject.QtDBus.QtTypeName.Out0\"/>\n"
"    </signal>\n"
"  </interface>\n"
        "")
public:
    CacheAdaptor(QObject *parent);
    ~CacheAdaptor() override;

public: // PROPERTIES
public Q_SLOTS: // METHODS
    QSpiAccessibleCacheArray GetItems();
Q_SIGNALS: // SIGNALS
    void AddAccessible(const QSpiAccessibleCacheItem &nodeAdded);
    void RemoveAccessible(const QSpiObjectReference &nodeRemoved);
};

} // end of namespace QtGuiPrivate

#endif
