#ifndef QT_FEATURES_Widgets_src_widgets_qtwidgets_config_p_h_H
#define QT_FEATURES_Widgets_src_widgets_qtwidgets_config_p_h_H

#define QT_FEATURE_gtk3 -1

#define QT_FEATURE_style_fusion 1

#define QT_FEATURE_style_mac -1

#define QT_FEATURE_style_windows 1

#define QT_FEATURE_style_windowsvista -1

#define QT_FEATURE_style_windows11 -1

#define QT_FEATURE_style_android -1

#define QT_FEATURE_effects 1

#define QT_FEATURE_widgettextcontrol 1


#endif // QT_FEATURES_Widgets_src_widgets_qtwidgets_config_p_h_H
