// Copyright (C) 2013 Klaralvdalens Datakonsult AB (KDAB)
// SPDX-License-Identifier: LicenseRef-Qt-Commercial OR LGPL-3.0-only OR GPL-2.0-only OR GPL-3.0-only
/***************************************************************************
** This file was generated by glgen version 0.1
** Command line was: glgen
**
** glgen is Copyright (C) 2012 Klaralvdalens Datakonsult AB (KDAB)
**
** This is an auto-generated file.
** Do not edit! All changes made to it will be lost.
**
****************************************************************************/

#ifndef QOPENGLVERSIONFUNCTIONS_P_H
#define QOPENGLVERSIONFUNCTIONS_P_H

//
//  W A R N I N G
//  -------------
//
// This file is not part of the Qt API.  It exists purely as an
// implementation detail.  This header file may change from version to
// version without notice, or even be removed.
//
// We mean it.
//

#include "qopenglversionfunctions.h"

#include <QtGui/private/qopenglcontext_p.h>

#include <QtOpenGL/qtopenglglobal.h>
#include <QtOpenGL/QOpenGLVersionProfile>
#include <QtCore/QSet>

QT_BEGIN_NAMESPACE

class QAbstractOpenGLFunctions;

class QOpenGLContextVersionData : public QOpenGLContextVersionFunctionHelper
{
public:
    QHash<QOpenGLVersionProfile, QAbstractOpenGLFunctions *> functions;
    QOpenGLVersionFunctionsStorage functionsStorage;
    QSet<QAbstractOpenGLFunctions *> externalFunctions;
    ~QOpenGLContextVersionData() override;
    static QOpenGLContextVersionData *forContext(QOpenGLContext *context);
};

QT_END_NAMESPACE

#endif // QOPENGLVERSIONFUNCTIONS_P_H
