// Copyright (C) 2016 Intel Corporation.
// SPDX-License-Identifier: LicenseRef-Qt-Commercial OR LGPL-3.0-only OR GPL-2.0-only OR GPL-3.0-only

#ifndef QTXMLGLOBAL_H
#define QTXMLGLOBAL_H

#include <QtCore/qglobal.h>
#include <QtXml/qtxml-config.h>
#include <QtXml/qtxmlexports.h>

#endif // QTXMLGLOBAL_H
