// Copyright (C) 2016 Intel Corporation.
// SPDX-License-Identifier: LicenseRef-Qt-Commercial OR LGPL-3.0-only OR GPL-2.0-only OR GPL-3.0-only

#ifndef QTOPENGLGLOBAL_H
#define QTOPENGLGLOBAL_H

#include <QtCore/qglobal.h>
#include <QtGui/qtguiglobal.h>
#include <QtOpenGL/qtopenglexports.h>

#endif // QTOPENGLGLOBAL_H
