// Copyright (C) 2023 The Qt Company Ltd.
// SPDX-License-Identifier: LicenseRef-Qt-Commercial OR BSD-3-Clause

// This file was generated by the qt_add_resources command.
// Qt-Security score:significant reason:default

#include <QtCore/qtsymbolmacros.h>

QT_DECLARE_EXTERN_RESOURCE(@RESOURCE_NAME@)

namespace {
    struct resourceReferenceKeeper {
        resourceReferenceKeeper() { QT_KEEP_RESOURCE(@RESOURCE_NAME@) }
    } resourceReferenceKeeperInstance;
}
