.gsc-control-cse table, table td, table th {
  border: none !important;
  margin-bottom: 10px !important
}
.gsc-control-cse {
  width: 100% !important;
  box-sizing: border-box !important
}
.gsc-control-cse * {
  font-family: 'Open Sans', Arial, Helvetica, sans-serif !important;
  line-height: 1.5 !important;
  font-weight: 300 !important
}
.gsc-control-cse,
.gsc-control-cse .gsc-table-result {
  width: 100% !important;
  font-family: 'Open Sans', Arial, Helvetica, sans-serif !important;
  font-weight: 300 !important;
  font-size: 13px !important
}
.gsc-resultsHeader {
  width: 100% !important;
  clear: both !important
}
.gsc-resultsHeader td.gsc-twiddleRegionCell {
  width: 75% !important
}
.gsc-resultsbox-visible {
  display: block !important
}
.gsc-resultsbox-invisible {
  display: none !important
}
.gsc-results {
  padding-bottom: 2px !important;
  width: 99% !important
}
.gsc-result {
  margin-bottom: 10px !important
}
.gsc-result .gs-title {
  height: 1.4em !important;
  overflow: hidden !important
}
.gsc-result div.gs-watermark {
  display: none !important
}
.gsc-results .gsc-result img.gs-ad-marker {
  display: none !important
}
.gsc-webResult:after {
  content: "." !important;
  display: block !important;
  height: 0 !important;
  clear: both !important;
  visibility: hidden !important
}
.gsc-webResult {
  zoom: 1 !important
}
.gsc-webResult .gsc-result {
  margin: 0 !important;
  padding: .5em 0 !important;
  border-bottom: 1px solid #ebebeb !important
}
.gsc-above-wrapper-area {
  border-bottom: 1px solid #E9E9E9 !important;
  padding: 5px 0 5px 0 !important
}
.gsc-above-wrapper-area-invisible {
  display: none !important
}
.gsc-above-wrapper-area-container {
  width: 100% !important
}
.gsc-result-info {
  text-align: left !important;
  color: #676767 !important;
  font-size: 13px !important;
  padding-left: 8px !important;
  margin: 10px 0 10px 0 !important
}
.gsc-result-info-container {
  text-align: left !important
}
.gsc-result-info-invisible {
  display: none !important
}
.gsc-orderby-container {
  text-align: right !important;
  background: transparent !important
}
.gsc-orderby-invisible {
  display: none !important
}
.gsc-orderby-label {
  color: #676767 !important;
  padding: 5px 5px 6px 0 !important
}
.gsc-selected-option-container {
  background-color: transparent !important;
  border: 1px solid #eee !important;
  border: 1px solid rgba(0, 0, 0, 0.1) !important;
  border-radius: 2px !important;
  box-shadow: none !important;
  color: #444 !important;
  cursor: default !important;
  font-size: 11px !important;
  font-weight: bold !important;
  height: 20px !important;
  line-height: 20px !important;
  max-width: 90% !important;
  min-width: 54px !important;
  outline: 0 !important;
  padding: 0 28px 0 6px !important;
  position: relative !important;
  text-align: center !important;
  width: 50px !important !important
}
.gsc-selected-option {
  position: relative !important;
  width: 100% !important;
}
.gsc-control-cse .gsc-option-selector {
  border: none !important;
  height: 11px !important;
  margin-top: -4px !important;
  position: absolute !important;
  right: 5px !important;
  top: 8px !important;
  width: 7px !important;
  padding: 0 5px 0 5px !important;
  background: url(//ssl.gstatic.com/ui/v1/disclosure/small-grey-disclosure-arrow-down.png) center no-repeat
}
.gsc-results .gsc-cursor-box .gsc-trailing-more-results {
  margin-bottom: 0 !important;
  display: inline !important
}
.gsc-results .gsc-cursor {
  display: inline !important
}
.gsc-results .gsc-cursor-box {
  margin: 10px 5px 10px !important
}
.gsc-results .gsc-cursor-box .gsc-cursor-page {
  cursor: pointer !important;
  color: #000000 !important;
  text-decoration: underline !important;
  margin-right: 8px !important;
  display: inline !important
}
.gsc-results .gsc-cursor-box .gsc-cursor-current-page {
  cursor: default !important;
  color: white !important;
  background-color: #5caa15 !important;
  font-weight: bold !important;
  text-decoration: none !important;
  padding: 0 3px 0 3px !important;
  margin-top: 10px !important;
  cursor: pointer !important
}
.gs-result .gs-title,
.gs-result .gs-title * {
  color: #5caa15 !important;
  text-decoration: underline !important
}
.gs-webResult div.gs-visibleUrl-long,
.gs-promotion div.gs-visibleUrl-long {
  overflow: hidden !important;
  display: none !important
}
.gs-webResult div.gs-per-result-labels a.gs-label {
  text-decoration: underline !important;
  cursor: pointer !important;
  padding: 3px !important;
  color: #26282a !important
}
.gs-webResult div.gs-per-result-labels a.gs-label.gs-labelActive {
  cursor: default !important;
  text-decoration: none !important
}
.gsc-control-cse:after {
  content: "." !important;
  display: block !important;
  clear: both !important;
  height: 0 !important;
  visibility: hidden !important
}

*:first-child + html .gsc-inline-block {
  display: inline !important
}
