#ifndef QT_FEATURES_GlobalConfig_src_corelib_global_qconfig_p_h_H
#define QT_FEATURES_GlobalConfig_src_corelib_global_qconfig_p_h_H

#define QT_FEATURE_use_bfd_linker -1

#define QT_FEATURE_use_gold_linker -1

#define QT_FEATURE_use_lld_linker -1

#define QT_FEATURE_use_mold_linker -1

#define QT_FEATURE_android_style_assets -1

#define QT_FEATURE_gc_binaries -1

#define QT_FEATURE_developer_build -1

#define QT_FEATURE_private_tests -1

#define QT_FEATURE_doc_snippets -1

#define QT_FEATURE_debug -1

#define QT_FEATURE_elf_private_full_version -1

#define QT_FEATURE_reduce_exports 1

#define QT_FEATURE_no_direct_extern_access -1

#define QT_FEATURE_x86intrin 1

#define QT_FEATURE_sse2 1

#define QT_FEATURE_sse3 1

#define QT_FEATURE_ssse3 1

#define QT_FEATURE_sse4_1 1

#define QT_FEATURE_sse4_2 1

#define QT_FEATURE_avx 1

#define QT_FEATURE_f16c 1

#define QT_FEATURE_avx2 1

#define QT_FEATURE_avx512f 1

#define QT_FEATURE_avx512er 1

#define QT_FEATURE_avx512cd 1

#define QT_FEATURE_avx512pf 1

#define QT_FEATURE_avx512dq 1

#define QT_FEATURE_avx512bw 1

#define QT_FEATURE_avx512vl 1

#define QT_FEATURE_avx512ifma 1

#define QT_FEATURE_avx512vbmi 1

#define QT_FEATURE_avx512vbmi2 1

#define QT_FEATURE_aesni 1

#define QT_FEATURE_vaes 1

#define QT_FEATURE_rdrnd 1

#define QT_FEATURE_rdseed 1

#define QT_FEATURE_shani 1

#define QT_FEATURE_lsx -1

#define QT_FEATURE_lasx -1

#define QT_FEATURE_mips_dsp -1

#define QT_FEATURE_mips_dspr2 -1

#define QT_FEATURE_neon -1

#define QT_FEATURE_arm_crc32 -1

#define QT_FEATURE_arm_crypto -1

#define QT_FEATURE_arm_sve -1

#define QT_FEATURE_localtime_r 1

#define QT_FEATURE_localtime_s -1

#define QT_FEATURE_posix_fallocate 1

#define QT_FEATURE_force_system_libs -1

#define QT_FEATURE_force_bundled_libs -1

#define QT_FEATURE_system_zlib 1

#define QT_FEATURE_stdlib_libcpp -1

#define QT_FEATURE_dbus 1

#define QT_FEATURE_dbus_linked 1

#define QT_FEATURE_gui 1

#define QT_FEATURE_network 1

#define QT_FEATURE_printsupport 1

#define QT_FEATURE_sql 1

#define QT_FEATURE_testlib 1

#define QT_FEATURE_widgets 1

#define QT_FEATURE_xml 1

#define QT_FEATURE_libudev 1

#define QT_FEATURE_openssl 1

#define QT_FEATURE_dlopen 1

#define QT_FEATURE_relocatable 1

#define QT_FEATURE_intelcet 1

#define QT_FEATURE_glibc_fortify_source 1

#define QT_FEATURE_trivial_auto_var_init_pattern 1

#define QT_FEATURE_stack_protector 1

#define QT_FEATURE_stack_clash_protection 1

#define QT_FEATURE_libstdcpp_assertions 1

#define QT_FEATURE_libcpp_hardening -1

#define QT_FEATURE_relro_now_linker 1

#define QT_FEATURE_android_16kb_pages -1


#endif // QT_FEATURES_GlobalConfig_src_corelib_global_qconfig_p_h_H
