// Copyright (C) 2022 The Qt Company Ltd.
// SPDX-License-Identifier: LicenseRef-Qt-Commercial OR LGPL-3.0-only OR GPL-2.0-only OR GPL-3.0-only
// Qt-Security score:significant reason:default

#ifndef QVARIANTMAP_H
#define QVARIANTMAP_H

#if 0
#pragma qt_class(QVariantMap)
#endif

#include <QtCore/qglobal.h>

#include <QtCore/QMap>
#include <QtCore/QVariant>
#include <QtCore/QString>

QT_BEGIN_NAMESPACE
QT_END_NAMESPACE

#endif
